#!/usr/bin/python

import os
import json
import getopt
import sys
import time
import subprocess
import operator
import configparser
import urllib.parse
import tempfile
import urllib3
urllib3.disable_warnings()
from proxmoxer import ProxmoxAPI

sys.tracebacklimit = 0

#################
def find_template_on_node(proxmox,node,name):
    for vm in proxmox.nodes(node).qemu.get():
        if (vm['template'] == 1 and vm['name'] == name):
            return vm['vmid']
    for vm in proxmox.nodes(node).lxc.get():
        if (vm['template'] == 1 and vm['name'] == name):
            return vm['vmid']
    return False

def find_vmid_by_name(proxmox,name):
    for node in proxmox.nodes.get():
        for vm in proxmox.nodes(node['node']).qemu.get():
            if (vm['name'] == name):
                return vm['vmid']
        for vm in proxmox.nodes(node['node']).lxc.get():
            if (vm['name'] == name):
                return vm['vmid']
    return False

def get_vms(proxmox,nodes):
    vms = []
    for node in nodes:
        for vm in proxmox.nodes(node['node']).qemu.get():
            vm['node'] = node['node']
            vm['type'] = "qemu"
            vms.append(vm)
        for vm in proxmox.nodes(node['node']).lxc.get():
            vm['node'] = node['node']
            vm['type'] = "lxc"
            vm['vmid'] = int(vm['vmid'])
            vms.append(vm)
    return vms

def do_action(proxmox,vm,action):
    return eval("proxmox.nodes.{}.{}({}).status.{}.post()".format(vm['node'],vm['type'],vm['vmid'],action))

def main() -> int:
    #################
    ## presets
    #################
    
    action = 'list'
    vmid = int() 
    name = ''
    nouser = 1 
    node = None
    server = None
    user = None
    password = None
    template = None
    list = False
    nodes = []
    configfile = '{}/.config/proxmox_api/config.ini'.format(os.environ.get('HOME'))
    
    #################
    ## options
    #################
    
    options, remainder = getopt.getopt(sys.argv[1:], 'n:s:u:p:v:a:t:o:d:c:hl', ['node=','server=','user=','password=','vmid=','action=','template=','nouser=','name=','config=','help','list'])
    
    for opt, arg in options:
        if opt in ('-n', '--node'):
            node = arg
        if opt in ('-s', '--server'):
            server = arg
        if opt in ('-u', '--user'):
            user = arg
        if opt in ('-p', '--password'):
            password = arg
        elif opt in ('-v', '--vmid'):
            vmid = int(arg)
        elif opt in ('-a','--action'):
            action = arg
        elif opt in ('-t','--template'):
            template = arg
        elif opt in ('-o','--nouser'):
            user = int(arg)
        elif opt in ('-d','--name'):
            name = arg
        elif opt in ('-c','--config'):
            configfile = arg
        elif opt in ('-l','--list'):
            list = True
        elif opt in ('-h','--help'):
            print("Access Proxmox by using API")
            print("-s, --server   - server to connect to")
            print("-u, --user     - login user")
            print("-p, --password - login password")
            print("-n, --node     - node to access")
            print("-v  --vmid     - vmid to use")
            print("-a, --action   - action to perform (list,create,start,stop,reset,shutdown,destroy,enter,view,vnc,subscription)")
            print("-t, --template - name of the template to use")
            print("-o, --nouser   - number of users")
            print("-d, --name     - search name instead of vmid")
            print("-l, --list     - list templates")
            print("-h, --help     - this help")
            return 0

    config = configparser.ConfigParser()
    if configfile != "/dev/null":
        if os.path.isfile(configfile):
            config.read(configfile)
        else:
            print(f"configuration file {configfile} was not found")
            return 1
    else:
        config['General'] = {}

    if 'General' not in config:
        print(f"configuration file {configfile} has no [General] section")
        return 1

    if server is not None:
        config['General']['server'] = server
    if user is not None:
        config['General']['user'] = user
    if password is not None:
        config['General']['password'] = password

    if 'server' not in config['General']:
        print(f"configuration file {configfile} has no 'server' key in [General] section")
        return 1
    if 'user' not in config['General']:
        print(f"configuration file {configfile} has no 'user' key in [General] section")
        return 1
    if 'password' not in config['General']:
        print(f"configuration file {configfile} has no 'password' key in [General] section")
        return 1
    if 'verify_ssl' not in config['General']:
        config['General']['verify_ssl'] = "False"


    if ( list == True ):
        for each_section in config.sections():
            if ( each_section.startswith('template:') ):
                print(each_section.replace('template:',''))
        return 0
    
    proxmox = ProxmoxAPI(config['General']['server'], user=config['General']['user'],
                         password=config['General']['password'],verify_ssl=config.getboolean('General','verify_ssl'))
    
    for node in proxmox.nodes.get():
        if node['status'] == 'online':
            nodes.append(node)
    
    vms   = get_vms(proxmox,nodes)
    
    
    
    if ( action == 'list' ):
        vms.sort(key=operator.itemgetter('vmid'))
        for vm in vms:
            print( "{0}@{1} {2} => {3}".format(vm['vmid'], vm['node'], vm['name'], vm['status']))
        return 0    
    if action == 'subscription':
        subscriptions = []
        for node in nodes:
            sub = proxmox.nodes(node['node']).subscription.get()
            sub['node'] = node['node']
            subscriptions.append(sub)
        print(json.dumps(subscriptions))
        return 0
    
    if ( name != ''):
        vmid = find_vmid_by_name(proxmox,name)
    if vmid == None or vmid == '':
      print("vmid not set")
      return 1
    
    vmid = int(vmid)
    
    if action == 'start':
        for vm in vms:
            if vm['vmid'] == vmid:
                print("starting {0}@{1}".format(vmid,vm['node']))
                do_action(proxmox,vm,action)
    
    if action == 'stop':
        for vm in vms:
            if vm['vmid'] == vmid:
                print("stopping {0}@{1}".format(vmid,vm['node']))
                do_action(proxmox,vm,action)
    
    if action == 'reset':
        for vm in vms:
            if vm['vmid'] == vmid:
                print("resetting {0}@{1}".format(vmid,vm['node']))
                do_action(proxmox,vm,action)
    
    if ( action == 'shutdown' ):
            for vm in vms:
                if (vm['vmid'] == vmid):
                    print("shutdown {0}@{1}".format(vmid,vm['node']))
                    do_action(proxmox,vm,action)
    
    if ( action == 'view' ):
        for vm in vms:
            if ((vmid == "" and vm['type'] == 'qemu') or (vm['vmid'] == vmid and vm['type'] == 'qemu')):
                if vm['status'] == 'stopped':
                    do_action(proxmox,vm,'start')
                    time.sleep(2)
                spice = proxmox.nodes(vm['node']).qemu(vm['vmid']).spiceproxy.post()
                with tempfile.NamedTemporaryFile(mode='w+b', suffix='.vv', delete=False) as file:
                  file.write(b"[virt-viewer]\n")
                  for key in spice:
                    file.write("{0}={1}\n".format(key,spice[key]).encode('utf-8'))
                  filename = file.name
                p = subprocess.Popen(['remote-viewer', '-f', filename])
    
    if ( action == 'vnc' ):
        for vm in vms:
            if (vm['vmid'] == vmid):
                config = { 'generate-password': 1}
                vnc = proxmox.nodes(vm['node']).qemu(vm['vmid']).vncproxy.post(**config)
                # for key in vnc:
                #    print("{0}={1}".format(key,vnc[key]))
                wss = proxmox.nodes(vm['node']).qemu(vm['vmid']).vncwebsocket.get(vncticket=vnc['ticket'],port=vnc['port'])
                # for key in wss:
                #    print("{0}={1}".format(key,wss[key]))
                url = 'https://{0}:8006/?console=kvm&novnc=1&node={1}&resize=scale&vmid={2}&vncticket={6}'.format(vm['node']+'.mylinuxtime.de',vm['node'],vmid,vm['node'],vmid,wss['port'],urllib.parse.quote(vnc['ticket'],safe=''))
                p = subprocess.Popen(['google-chrome-stable', url])
    
    if ( action == 'enter' ):
            for vm in vms:
                if (vm['vmid'] == vmid):
                    print("enter {0}@{1}".format(vmid,vm['node']))
                    os.system("ssh -t root@{0}.mylinuxtime.de pct enter {1}".format(vm['node'],vmid))
    
    if ( action == 'destroy' ):
            for vm in vms:
                if (vm['vmid'] == vmid):
                    if (vm['status'] == 'stopped'):
                        print("destroying {0}@{1}".format(vmid,vm['node']))
                        eval("proxmox.nodes.{}.{}({}).delete()".format(vm['node'],vm['type'],vmid))
                    else:
                        print("cannot destroy {0}@{1} status is {2}".format(vmid,vm['node'],vm['status']))
    
    if action == 'create':
        source = find_template_on_node(proxmox,node,template)
        mac_high = int(int(vmid)/256)
        mac_low  = int(vmid)-mac_high*256
        if source:
            print("creating {0}@{1}".format(vmid,node))
            newcontainer = { 'newid': vmid,
                'description': "Schulung%02d - %s" % (nouser,template),
                'full': config['template:{}'.format(template)]['full'],
                'name': "schulung%02d-%s" % (nouser,template),
                'pool': "Schulung%02d" % (nouser) }
            mynode = proxmox.nodes(node)
            mynode.qemu(source).clone.post(**newcontainer)
            containerconfig = { 'balloon': config['template:{}'.format(template)]['balloon'],
                'memory': config['template:{}'.format(template)]['memory'],
                'sockets': config['template:{}'.format(template)]['sockets'],
                'cores': config['template:{}'.format(template)]['cores'],
                'serial0': config['template:{}'.format(template)]['serial0'],
                'net0': "%s,macaddr=%s:%02x:%02x" % (config['template:{}'.format(template)]['net0'],config['template:{}'.format(template)]['macbase'],mac_high,mac_low)
                }
            mynode.qemu(vmid).config.post(**containerconfig)
        else:
            print("no template for {0} on node {1}".format(template,node))

if __name__ == '__main__':
    sys.exit(main())
